import json
import subprocess
from typing import List

import click
from azure.common.credentials import get_azure_cli_credentials

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

@click.command(context_settings=CONTEXT_SETTINGS)
@click.option("-n", required=False, type=int, help="Switch to this subscription number directly.")
@click.option("-v", is_flag=True, help="Verbose: echo the azure-cli commands.")
def main(n: int = None, v: bool = False) -> None:
    """
    Show all Azure Subscriptions in current profile using the `az` command-line utility.
    Ask user input for switching to another subscription.
    """
    try:
        # Using --query to map subset of fields and sort by name (ascending)
        list_cmd = 'az account list --all --output json ' \
            '--query "sort_by([].{name:name, isDefault:isDefault, id:id, state:state}, &name)"'
        if v:
            click.echo(f'Issuing AZ CLI command: {list_cmd}')
        subscriptions = json.loads(subprocess.getoutput(list_cmd))

        current_nr = _print_options(subscriptions)

        if not n:
            n = click.prompt('Switch', type=int, default=current_nr)

        if n not in range(1, len(subscriptions) + 1):
            raise ValueError("Value not in range! Not changing subscription.")

        if n == current_nr:
            click.echo("Selection is same as current. Not changing subscription.")
        else:
            _select_subscription(n, v, subscriptions)

        _, subscription_id = get_azure_cli_credentials()
        active = next(filter(lambda x: x['id'] == subscription_id, subscriptions))
        click.echo("Active: " + click.style(active['id'] + ": " + active['name'], fg='green', bold=True))

        if active['state'].lower() == 'disabled':
            click.echo(click.style("Subscription state is Disabled, requires: az login!", fg='yellow'))

    except subprocess.CalledProcessError:
        # Issue is already printed to stderr.
        pass
    except click.exceptions.Abort:
        # No need to raise exception when CTRL-C out of the cli
        pass
    except ValueError as e:
        print(e)


def _select_subscription(n, v, subscriptions):
    subscription_id = subscriptions[n - 1]['id']
    # replaced shell=true variant, which is more vulnerable: https://stackoverflow.com/a/29023432
    switch_cmd = f'az account set -s {subscription_id}'
    if v:
        click.echo(f'Issuing AZ CLI command: "{switch_cmd}"')
    subprocess.check_output(switch_cmd.split(" "))


def _print_options(subscriptions: List[dict]) -> int:
    selected = -1

    for idx, s in enumerate(subscriptions):
        number = click.style(f"[{idx + 1}]", fg='yellow')

        if s['isDefault']:
            selected = idx + 1
            colored_info = click.style(f"{s['id']}: {s['name']}", fg='green', bold=True)
        else:
            colored_info = click.style(f"{s['id']}: {s['name']}")

        click.echo(number + ": " + colored_info)
    return selected

